; ------------------------------------------------------
; Einfgen von Parametern 
; ------------------------------------------------------

	JMP 	Start	; berspringe die Datentabelle

	DB	0	; Reserviere den Speicherplatz [02]
	DB	0	; Reserviere den Speicherplatz [03]

Start:  MOV	AL,5
	MOV	BL,4
	CALL	30	; Ein Prozess, um AL und BL zu addieren
			; Das Ergebnis geht nach AL zurck.

; ----- Benutze RAM-Stellen, um Parameter in einen Prozess einzufgen --
	MOV	AL,3
	MOV	[02],AL	; Speichere 3 unter der Adresse [02]
	MOV	BL,1
	MOV	[03],BL	; Speichere 1 unter der Adresse [03]
	CALL	40

; ----- Benutze den Stapelspeicher, um Parameter in einen Prozess einzufgen ------
	MOV	AL,7
	PUSH	AL
	MOV	BL,2
	PUSH	BL
	CALL	60
	POP	BL
	POP	AL	; Dieser Punkt enthlt die Antwort.

	JMP	Start	; Geh zurck und fang von vorne an.

; ----- Eine Prozedur, um zwei Zahlen zu addieren -------------------------
;       Die Parameter werden in den Prozess eingefgt, indem man AL und BL verwendet.
;       Das Ergebnis geht zurck nach AL.
;       Diese Methode ist einfach, aber ungeeignet, wenn viele Parameter 
;       eingefgt werden mssen.

	ORG	30	; Der Code beginnt unter der Adresse [30].

	ADD	AL,BL	; Fhre die Addition aus. Das Ergebnis geht nach AL.

	RET		; Zieh dich aus dem Prozess zurck.

; ----- Eine weitere Prozedur, um zwei Zahlen zu addieren -------------------------
;       Hier werden die Parameter mit Hilfe von RAM-Stellen eingefgt.
;       Das Ergebnis geht zurck in die RAM-Stelle.
;       Diese Methode ist komplizierter, aber es gibt keine Begrenzung bei
; 	der Zahl von einzufgenden Parametern, es sei denn, der RAM hat 
;	keinen Platz mehr.

	ORG	40	; Der Code beginnt bei der Adresse [40].

	PUSH	CL	; Speichere die Register und die Flags im Stapelspeicher.
	PUSH	DL
	PUSHF

	MOV	CL,[02]	; Hole einen Parameter aus RAM.
	MOV	DL,[03]	; Hole einen Parameter aus RAM.
	ADD	CL,DL	; Fhre die Addition aus.
	MOV	[02],CL	; Speichere das Ergebnis in RAM.

	POPF		; Stelle das ursprngliche Register und die
	POP	DL	; Werte der Flags wieder her.
	POP	CL

	RET

; ----- Ein weiterer Prozess, um zwei Zahlen zu addieren -------------------------
; 	Die zu addierenden Zahlen liegen im Stapelspeicher.
;	POP Die Parameter werden herausgeholt.
;	Fhre die Addition aus.
;	Bringe die Antwort zurck auf den Stapelspeicher.
;	
        Der grte Teil der Prozessaufrufe im wirklichen Leben benutzt
;	den Stapelspeicher fr das Einfgen von Parametern. 
;	Es ist durchaus blich,
;	dass die Adresse einer komplizierten Datenstruktur in RAM in einen 
;	Prozess eingefgt wird, der den Stapelspeicher benutzt.

	ORG	60	; Der Code beginnt bei der Adresse [60]

	POP	DL	; Zurck zur Adresse
	POP	BL	; Ein Parameter
	POP	AL	; Ein Parameter

	ADD	AL,BL

	PUSH	AL	; Antwort ; Die Anzahl der Verschiebungen muss
	PUSH	AL	; zu der Anzahl der Pops passen.
	PUSH	DL	; Stelle den Stapelspeicher wieder so her, 
			; wie er vorher war.

	RET
; --------------------------------------------------------------
	END
